/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.lang.reflect.Method;
import javax.ejb.EJBException;

import org.jfox.ejb.connector.ClientContainerInvoker;
import org.jfox.ejb.connector.ConnectorHelper;
import org.jfox.ejb.connector.ContainerRemote;
import org.jfox.ejb.connector.local.LOCALContainerInvoker;
import org.jfox.ejb.connector.local.LOCALContainerService;
import org.jfox.ejb.meta.Protocol;
import org.jfox.framework.Registry;

/**
 * DelegateInvocationHandler һЭ޹صInvocationHandler EJBHome 󶨵 jndi ϣ
 *  ejbHome.create ʱ򣬽 setProtocolInvocationHandler һЭ InvocationHandler
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class DelegateInvocationHandler implements ClientContainerInvoker {

    // remote object identity
    private EJBObjectId objectId;
    private Protocol defualtRemoteProtocol;
    private Protocol defualtLocalProtocol;
    private ClientContainerInvoker protocolInvocationHandler = null;

    public DelegateInvocationHandler(EJBObjectId objectId, Protocol remoteProtocol, Protocol localProtocol) {
        this.objectId = objectId;
        this.defualtRemoteProtocol = remoteProtocol;
        this.defualtLocalProtocol = localProtocol;
    }

    public EJBObjectId getObjectId() {
        return objectId;
    }

    public ClientContainerInvoker getProtocolInvocationHandler() {
        return protocolInvocationHandler;
    }

    public void setProtocolInvocationHandler(ClientContainerInvoker protocolInvocationHandler) {
        this.protocolInvocationHandler = protocolInvocationHandler;
    }

    /**
     * ʵͨõ Object 磺toString, hashCode 
     *
     * @param proxy
     * @param method
     * @param args
     * @return
     * @throws Throwable
     */
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        /*
         * ѡһЭ InvocationHandler useProtocol ָ
         */
        if(method.equals(ExtendedEJBHomeMethod.UseProtocol)) {
            Protocol protocol = (Protocol) args[0];
            try {
                protocolInvocationHandler = getInvocationHandlerByProtocol(protocol);
            }
            catch(EJBException e) {
                throw e;
            }
            catch(Exception e) {
                throw new EJBException("protocol " + protocol + " not supported!", e);
            }
            return null; // õǡuseProtocol Ｔ
        }

        // û protocolInvocationHandlerʹĬϵ InvocationHandler
        if(protocolInvocationHandler == null) {
            synchronized(this) {
                protocolInvocationHandler = getInvocationHandlerByDefault();
            }
        }
        // delegate invocation to Э InvocationHandler
        return protocolInvocationHandler.invoke(proxy, method, args);
    }

    private ClientContainerInvoker getInvocationHandlerByProtocol(Protocol protocol) throws Exception {
        if(protocol.equals(Protocol.LOCAL)) {
            // ʹáLocal ЭʱҪԴȷBeanContainerͬһJVM
/*      if(!ObjectId.CONTAINER_IDENTITY.equals(objectId.getIdentity())){
        throw new EJBException("can not use local protocol, because the bean is not in same JVM as Container");
      }*/

            if(!objectId.isCurrentVM()) {
                throw new EJBException("can not use local protocol, because the bean is not in same JVM as Container");
            }
            else {
//        return new LOCALContainerInvoker(objectId,LOCALContainerService.getInstance());
                return new LOCALContainerInvoker(objectId, (ContainerRemote) (Registry.getInstance().getFirstComponentMetabyImplemetation(LOCALContainerService.class).getComponentInstance()));
            }
        }
        else {
            // will use jndi to lookup default JRMPContainerService
            ContainerRemote invoker = ConnectorHelper.lookupContainerInvoker(protocol);
            return ConnectorHelper.createClientInvocationHandler(protocol, objectId, invoker);
        }
    }

    private ClientContainerInvoker getInvocationHandlerByDefault() throws Exception {
        // CONTAINER_IDENTITY  Ҫõ ejb  ObjectId  Identity һ£ ejb ڴ
//    if(ObjectId.CONTAINER_IDENTITY.equals(objectId.getIdentity())){
        if(objectId.isCurrentVM()) {
            //  EJB <-> EJB ķʣĬʹálocal Э
            return getInvocationHandlerByProtocol(defualtLocalProtocol);
        }
        else { // Զ̵ãĬʹ jrmp Э
            return getInvocationHandlerByProtocol(defualtRemoteProtocol);
        }
    }

}