/* JFox, the OpenSource J2EE Application Server
 *
 * Distributable under GNU LGPL license by gun.org
 * more details please visit http://www.huihoo.org/jfox
 */

package org.jfox.ejb;

import java.rmi.RemoteException;
import javax.ejb.EnterpriseBean;
import javax.ejb.Handle;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jfox.ejb.handler.HandlerChain;
import org.jfox.framework.ComponentName;
import org.jfox.framework.Registry;

/**
 * ִ EJBObject EJBHome жķ
 *
 * @author <a href="mailto:young_yy@hotmail.com">Young Yang</a>
 */

public class StatelessSessionBucket extends SessionBucket {

    private StatelessSessionObjectPool instancePool = null;
    /**
     * StatlessSessionBean ÿηͬObjectId
     */
    private EJBObjectId beanObjectId = null;
    private SessionContext ejbConext = null;

    public StatelessSessionBucket(BucketMetaData meta) {
        super(meta);
    }

    /**
     *  EJBHome ķ
     *
     * @param invocation
     * @return
     * @throws Exception
     */
    public Object invokeHome(Invocation invocation) throws Exception {
        logger.debug("invokeHome " + invocation.getMethod().getName());
        return ((HandlerChain) Registry.getInstance().getComponentInstance(ComponentName.parseString(StatelessHandlerChain.class.getName() + "@default"))).invokeHome(this, invocation);
    }

    /**
     * ִ Bean ķ
     *
     * @param invocation
     * @return
     * @throws Exception
     */
    public Object invokeBean(Invocation invocation) throws Exception {
        logger.debug("invokeBean " + invocation.getMethod().getName());
        return ((HandlerChain) Registry.getInstance().getComponentInstance(ComponentName.parseString(StatelessHandlerChain.class.getName() + "@default"))).invokeBean(this, invocation);

    }

    //  StatelessSessionBeanִκβ
    public void remove(Handle handle) throws RemoteException, RemoveException {
        try {
            EJBPoolableObject pobj = (EJBPoolableObject) instancePool.retrieveObject();
            remove(pobj.getBeanInstance());
        }
        catch(Exception e) {
            throw new RemoveException(e.getMessage());
        }
    }

    public void remove(EnterpriseBean bean) throws RemoteException, RemoveException {
        ejbRemove((SessionBean) bean);
    }

    //  StatelessSessionBean, Ƿ true
    public boolean isIdentical(EJBObjectId thisObjectId, EJBObjectId thatObjectId) throws RemoteException {
        return true;
    }

    public EJBPoolableObject retrieveBean(Object key) throws Exception {
        StatelessEJBPoolableObject pobj = (StatelessEJBPoolableObject) instancePool.retrieveObject();
        if(!pobj.isEjbCreated()) { // ĶҪ setSessionContext, ejbCreate
            SessionBean bean = (SessionBean) pobj.getBeanInstance();
            setSessionContext(bean, ejbConext);
            ejbCreate(bean, "create", null);
            pobj.ejbCreated();
        }
        return pobj;
    }

    public void restoreBean(Object key, EJBPoolableObject pobj) {
        instancePool.restoreObject(pobj);
    }

    public EJBObjectId createBean(Invocation invocation) throws Exception {
        if(beanObjectId == null) { // һε create  beanObjectId,ejbContext
            beanObjectId = nextObjectId();
            ejbConext = new SessionContextImpl(beanObjectId, bucketMeta);
        }
        StatelessEJBPoolableObject pobj = (StatelessEJBPoolableObject) instancePool.createObject();
        if(!pobj.isEjbCreated()) { // ĶҪ setSessionContext, ejbCreate
            SessionBean bean = (SessionBean) pobj.getBeanInstance();
            setSessionContext(bean, ejbConext);
            // stateless session bean ֻһûв ejbCreate 
            ejbCreate(bean, "create", null);
            pobj.ejbCreated();
        }
        this.restoreBean(invocation.getObjectId(), pobj);
        return beanObjectId;
    }

    protected void doInit() throws Exception {
        instancePool = new StatelessSessionObjectPool(new StatelessEJBObjectFactory(StatelessEJBPoolableObject.class, bucketMeta.getBeanClass(), this));
        instancePool.init();
        super.doInit();
    }

    protected void doDestroy() throws Exception {
        super.doDestroy();
        instancePool.destroy();
    }
}